# Copyright (c) 2015 by Miblo <miblodelcarpio@gmail.com>
# All rights reserved
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import weechat

weechat.register("weestreamer", "Miblo", "0.4.2", "GPL3", "Streamlink companion for WeeChat", "", "")

def stream(data, buffer, args):
    bufserver = weechat.buffer_get_string(weechat.current_buffer(), "localvar_server")
    bufchannel = weechat.buffer_get_string(weechat.current_buffer(), "localvar_channel").lstrip("#")
    quality = "best"

    input = args.split()
    if not input:
        server = bufserver
        channel = bufchannel
    elif len(input) == 1:
        server = bufserver
        channel = input[0]
    elif len(input) == 2:
        server = input[0]
        channel = input[1]
    else:
        weechat.prnt(weechat.current_buffer(), "%sToo many arguments (%s). Please see /help weestreamer"
                % (weechat.prefix("error"),(len(input))))
        return weechat.WEECHAT_RC_ERROR

    # NOTE(matt): https://streamlink.github.io/plugin_matrix.html
    servers = { "afreeca":"http://play.afreeca.com/%s" % (channel),
                "hitbox":"http://www.hitbox.tv/%s" % (channel),
                "twitch":"http://www.twitch.tv/%s" % (channel),
                "ustream":"http://www.ustream.tv/%s" % (channel.replace("-", ""))}

    streamurl = ""
    for key in servers.keys():
        if key in server:
            streamurl = servers[key]
    if not streamurl:
        weechat.prnt(weechat.current_buffer(), "%sUnsupported server: %s"
                % (weechat.prefix("error"), server))
        weechat.prnt(weechat.current_buffer(), "Currently supported servers:")
        for key in sorted(servers.keys()):
            weechat.prnt(weechat.current_buffer(), "    %s" % key)
        return weechat.WEECHAT_RC_ERROR

    command = "streamlink %s %s" % (streamurl, quality)

    weechat.prnt(weechat.current_buffer(), "%sLAUNCHING: %s" % (weechat.prefix("action"), command))
    weechat.hook_process("%s" % (command), 0, "handle_output", "")
    return weechat.WEECHAT_RC_OK

def handle_output(data, command, rc, out, err):
    global process_output
    process_output = ""
    if out != "":
        process_output += out
    if int(rc) >= 0:
        weechat.prnt(weechat.current_buffer(), process_output)
    return weechat.WEECHAT_RC_OK

weechat.hook_command("weestreamer", "Streamlink companion for WeeChat",
        "server channel",

        "Run /weestreamer without any arguments while in a channel on a supported irc\n"
        "server to launch the stream associated with that channel.\n"
        "\n"
        "You may optionally pass the server and / or channel (in that order) to launch\n"
        "the required stream from any channel, e.g.:\n"
        "    /weestreamer twitch handmade_hero\n"
        "    /weestreamer handmade_hero\n"
        "\n"
        "Currently supported servers:\n"
        "   afreeca\n"
        "   hitbox\n"
        "   twitch\n"
        "   ustream\n"
        "\n"
        "\n"
        "Troubleshooting: If you expect that your current server should be supported but\n"
        "weestreamer keeps erroring, please check the name of the server by running:\n"
        "\n"
        "    /buffer localvar\n"
        "\n"
        "If you have named the server such that it doesn't contain the string in\n"
        "\"Currently supported servers\" (above), weestreamer will not recognise it.",

        # NOTE(matt): list of valid parameters
        "afreeca"
        " || hitbox"
        " || twitch"
        " || ustream",
        "stream", "")



# ===============================================================
# weeechat-axolotl.py (c) 2014-2016 by David R. Andersen <k0rx@rxcomm.net>
# based on:
# crypt.py (c) 2008-2012 by Nicolai Lissner <nlissne@linux01.org>
# ===============================================================
SCRIPT_NAME    = "axolotl"
SCRIPT_AUTHOR  = "David R. Andersen <k0rx@rxcomm.net>"
SCRIPT_VERSION = "0.1.0"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "encrypt/decrypt PRIVMSGs using axolotl ratchet and GPG"

"""
This plugin uses the Axolotl ratchet protocol with
gnupg to encrypt/decrypt messages you send or
receive with weechat. The script is largely copied
from the weechat crypt.py script. Thanks to the authors
for that! The script requires the Axolotl python module.
This module is available at:
https://github.com/rxcomm/pyaxo

you can add 'axolotl' to  weechat.bar.status.items to
have an indication that the message you are going to send
is encrypted (i.e. a database exists).

Example usage: if your nick is thingone, and the nick you want
to pm privately with is thingtwo, you would
generate two database files thingone.db and thingtwo.db
using the gen_weechat_database_pair.py utility.
thingtwo.db would go in your weechat directory, and
thingone.db would go in thingtwo's weechat directory.

If your buddy wants to re-encrypt his database with a
different password, he can do that directly with gpg.

 echo <oldpassword> | gpg --passphrase-fd=0 -d thingone.db | gpg --output thingone.db --cipher-algo AES256 -c

For some reason gpg won't prompt for the new passphrase
the first time when you run the above command. Just go
ahead and enter it - it _will_ prompt you the second time.
When the two match, all is good.

Of course, you need to share this database with the
remote side in another secure way (i.e. sending
pgp-encrypted mail). If you prefer to have each user
generate his/her own database, you can use the
init_conversation.py utility in the pyaxo repo
linked above.

The latest version of the script and database utility
can be found at:
https://github.com/rxcomm/weechat-axolotl
"""


import weechat, string, os, re
from pyaxo import Axolotl
from binascii import b2a_base64, a2b_base64

script_options = {
    "message_indicator" : "(enc) ",
    "statusbar_indicator" : "(PFS encrypted) ",
}

def getPasswd(username):
    """modify as appropriate"""
    return username+'123'

def decrypt(data, msgtype, servername, args):
  global decrypted
  hostmask, chanmsg = string.split(args, "PRIVMSG ", 1)
  channelname, message = string.split(chanmsg, " :", 1)
  if re.match(r'^\[\d{2}:\d{2}:\d{2}]\s', message):
    timestamp = message[:11]
    message = message[11:]
  else:
    timestamp = ''
  if channelname[0] == "#":
    username=channelname
  else:
    username, rest = string.split(hostmask, "!", 1)
    username = username[1:]
  nick = channelname.strip()
  if os.path.exists(weechat_dir + '/' + username + '.db'):
    a = Axolotl(nick, dbname=weechat_dir+'/'+username+'.db', dbpassphrase=getPasswd(username))
    a.loadState(nick, username)
    decrypted = a.decrypt(a2b_base64(message))
    a.saveState()
    del a
    if decrypted == "":
      return args
    decrypted = ''.join(c for c in decrypted if ord(c) > 31 or ord(c) == 9 or ord(c) == 2 or ord(c) == 3 or ord(c) == 15)
    return hostmask + "PRIVMSG " + channelname + " :" + chr(3) + "04" + weechat.config_get_plugin("message_indicator") + chr(15) + timestamp + decrypted
  else:
    return args

def encrypt(data, msgtype, servername, args):
  global encrypted
  pre, message = string.split(args, ":", 1)
  prestr=pre.split(" ")
  username=prestr[-2]
  buf = weechat.current_buffer()
  nick = weechat.buffer_get_string(buf, 'localvar_nick')
  if os.path.exists(weechat_dir + '/' + username + '.db'):

    a = Axolotl(nick, dbname=weechat_dir+'/'+username+'.db', dbpassphrase=getPasswd(username))
    a.loadState(nick, username)
    encrypted = a.encrypt(message)
    if encrypted == '':
        return args
    encrypted = b2a_base64(encrypted)
    a.saveState()
    del a
    encrypted = encrypted.replace("\n","")
    final_msg = pre + ":" +encrypted
    if len(encrypted) > 400:
      # I arrived at this next equation heuristically. If it doesn't work, let me know
      # and I will work on it some more. -DRA
      numsplits = 2*int(len(encrypted)/400) + 1
      splitmsg=string.split(message," ")
      cutpoint=int(len(splitmsg)/numsplits)
      encrypted_list = []
      for i in range(numsplits+1):
        if min((i+1)*cutpoint, len(splitmsg)) == (i+1)*cutpoint:
          segment = string.join(splitmsg[i*cutpoint:(i+1)*cutpoint]," ") + "\n"
          a = Axolotl(nick, dbname=weechat_dir+'/'+username+'.db', dbpassphrase=getPasswd(username))
          a.loadState(nick, username)
          encrypted = b2a_base64(a.encrypt(segment))
          a.saveState()
          del a
          valid_segment = True
        else:
          segment = string.join(splitmsg[i*cutpoint:]," ")
          if segment.strip() is None or len(segment) == 0:
            valid_segment = False
          else:
            a = Axolotl(nick, dbname=weechat_dir+'/'+username+'.db', dbpassphrase=getPasswd(username))
            a.loadState(nick, username)
            encrypted = b2a_base64(a.encrypt(segment))
            a.saveState()
            del a
            valid_segment = True
        encrypted = encrypted.replace("\n","")
        if valid_segment:
          encrypted_list += [encrypted]
      final_msg = ''
      for item in encrypted_list:
        final_msg = final_msg + pre + ":" + item + '\n'
    return final_msg
    return encrypted
  else:
    return args

def update_encryption_status(data, signal, signal_data):
    buffer = signal_data
    weechat.bar_item_update('axolotl')
    return weechat.WEECHAT_RC_OK

def encryption_statusbar(data, item, window):
    if window:
      buf = weechat.window_get_pointer(window, 'buffer')
    else:
      buf = weechat.current_buffer()
    if os.path.exists(weechat_dir + '/' + \
         weechat.buffer_get_string(buf, 'short_name') + '.db'):
      return weechat.config_get_plugin("statusbar_indicator")
    else:
      return ""

# register plugin
if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, \
                    SCRIPT_LICENSE, SCRIPT_DESC, "", "UTF-8"):
    weechat_dir = weechat.info_get("weechat_dir","")
    key_dir = weechat.config_get_plugin('key_dir')
    version = weechat.info_get("version_number", "") or 0
    if int(version) < 0x00030000:
      weechat.prnt("", "%s%s: WeeChat 0.3.0 is required for this script."
              % (weechat.prefix("error"), SCRIPT_NAME))
    else:
      weechat.bar_item_new('axolotl', 'encryption_statusbar', '')
      for option, default_value in script_options.iteritems():
          if not weechat.config_is_set_plugin(option):
                  weechat.config_set_plugin(option, default_value)

      # register the modifiers
      weechat.hook_modifier("irc_in_privmsg", "decrypt", "")
      weechat.hook_modifier("irc_out_privmsg", "encrypt", "")
      weechat.hook_signal("buffer_switch","update_encryption_status","")



# -*- coding: utf-8 -*-
###
# Copyright (c) 2010 by Elián Hanisch <lambdae2@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
###

###
# Shows highest and lowest user count for joined channels,
# and an average (for a period of a month)
#
#
#   Commands:
#   * /chanstat
#     Prints current channel stats, see /help chanstat
#
#
#   Settings:
#   * plugins.var.python.chanstat.path:
#     path where to store stat files, deault '%h/chanstat'
#
#   * plugins.var.python.chanstat.averge_period:
#     Period of time for calculate the average stats. This means the avegare will be calculated with
#     the users present in the last x days. Default is 30 days.
#
#   * plugins.var.python.chanstat.show_peaks:
#     If 'on' it will display a message when there's a user peak in any channel.
#     Valid values: on, off
#
#   * plugins.var.python.chanstat.show_lows:
#     If 'on' it will display a message when there's a user low in any channel.
#     Valid values: on, off
#
#
#   History:
#   2010-06-08
#   version 0.1: initial release.
#
###

SCRIPT_NAME    = "chanstat"
SCRIPT_AUTHOR  = "Elián Hanisch <lambdae2@gmail.com>"
SCRIPT_VERSION = "0.1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "Channel statistics"

try:
    import weechat
    WEECHAT_RC_OK = weechat.WEECHAT_RC_OK
    import_ok = True
except ImportError:
    print "This script must be run under WeeChat."
    print "Get WeeChat now at: http://weechat.flashtux.org/"
    import_ok = False

import time
now = lambda : int(time.time())

time_hour = 3600
time_day  = 86400
time_year = 31536000

### messages
def debug(s, args=(), prefix='', name_suffix='debug', level=1):
    """Debug msg"""
    l = weechat.config_get_plugin('debug')
    if not (l and int(l) >= level): return
    buffer_name = '%s_%s' %(SCRIPT_NAME, name_suffix)
    buffer = weechat.buffer_search('python', buffer_name)
    if not buffer:
        buffer = weechat.buffer_new(buffer_name, '', '', '', '')
        weechat.buffer_set(buffer, 'nicklist', '0')
        weechat.buffer_set(buffer, 'localvar_set_no_log', '1')
    weechat.prnt(buffer, '%s\t%s' %(prefix, s %args))

def error(s, prefix=SCRIPT_NAME, buffer=''):
    """Error msg"""
    prefix = prefix or script_nick
    weechat.prnt(buffer, '%s%s %s' %(weechat.prefix('error'), prefix, s))

def say(s, prefix=None, buffer=''):
    """Normal msg"""
    prefix = prefix or script_nick
    weechat.prnt(buffer, '%s\t%s' %(prefix, s))

### config and value validation
boolDict = {'on':True, 'off':False}
def get_config_boolean(config):
    value = weechat.config_get_plugin(config)
    try:
        return boolDict[value]
    except KeyError:
        default = settings[config]
        error("Error while fetching config '%s'. Using default value '%s'." %(config, default))
        error("'%s' is invalid, allowed: 'on', 'off'" %value)
        return boolDict[default]

def get_config_int(config):
    value = weechat.config_get_plugin(config)
    try:
        return int(value)
    except ValueError:
        default = settings[config]
        error("Error while fetching config '%s'. Using default value '%s'." %(config, default))
        error("'%s' is not a number." %value)
        return int(default)

def get_dir(filename):
    import os
    basedir = weechat.config_get_plugin('path').replace('%h', weechat.info_get('weechat_dir', ''))
    if not os.path.isdir(basedir):
        os.makedirs(basedir)
    return os.path.join(basedir, filename.lower())


class CaseInsensibleString(str):
    def __init__(self, s=''):
        self.lowered = s.lower()

    def __eq__(self, s):
        try:
            return self.lowered == s.lower()
        except:
            return False

    def __ne__(self, s):
        return not self == s

    def __hash__(self):
        return hash(self.lowered)

def caseInsensibleKey(k):
    if isinstance(k, str):
        return CaseInsensibleString(k)
    elif isinstance(k, tuple):
        return tuple([ caseInsensibleKey(v) for v in k ])
    return k

class CaseInsensibleDict(dict):
    key = staticmethod(caseInsensibleKey)

    def __setitem__(self, k, v):
        dict.__setitem__(self, self.key(k), v)

    def __getitem__(self, k):
        return dict.__getitem__(self, self.key(k))

    def __delitem__(self, k):
        dict.__delitem__(self, self.key(k))

    def __contains__(self, k):
        return dict.__contains__(self, self.key(k))


class Channel(object):
    def __init__(self, max=None, min=None, max_date=None, min_date=None, avrg_date=None,
            avrg_period=None, average=None, count=0):
        if not max:
            max = count
        if not min:
            min = 0
        if not average:
            average = float(count)
        if not max_date:
            max_date = now()
        if not min_date:
            min_date = now()
        if not avrg_date:
            avrg_date = now()
        if not avrg_period:
            avrg_period = 0
        self.max = max
        self.min = min
        self.max_date = max_date
        self.min_date = min_date
        self.avrg_date = avrg_date
        self.avrg_period = avrg_period
        self.average = average

    def __iter__(self):
        return iter((self.max, self.min, self.max_date, self.min_date, self.avrg_date,
            self.avrg_period, self.average))

    def __str__(self):
        return 'Channel(max=%s, average=%s, min_delta=%s)' %(self.max, self.average, self.min)


class StatLog(object):
    def __init__(self):
        self.writers = CaseInsensibleDict()

    @staticmethod
    def make_log(key):
        return get_dir('%s_%s.cvs' %key)

    def log(self, key, *args):
        if key in self.writers:
            writer = self.writers[key]
        else:
            import csv
            filename = self.make_log(key)
            writer = csv.writer(open(filename, 'ab'))
            self.writers[key] = writer

        writer.writerow(args)

    def get_reader(self, key):
        if key in self.writers:
            del self.writers[key]
        import csv
        return csv.reader(open(self.make_log(key)))

    def close(self):
        self.writers = {}


class ChanStatDB(CaseInsensibleDict):
    def __init__(self):
        self.logger = StatLog()

    def __setitem__(self, key, value):
        if not value:
            return
        debug(' ** stats update for %s', args=key[1])
        _now = now()
        avrg = 0
        if key in self:
            chan = self[key]
            if value > chan.max:
                debug('PEAK, %s: %s', args=(key[1], value))
                chan.max = value
                new_channel_peak(key, value, chan.max_date)
                chan.max_date = _now
            elif (chan.max - value) > chan.min:
                # we save the difference between max and min rather the min absolute value, because
                # the minimum min value for a channel would be 1, and that's isn't interesting.
                min_delta = chan.max - value
                debug('LOW, %s: %s', args=(key[1], value))
                chan.min = min_delta
                new_channel_low(key, value, chan.min_date)
                chan.min_date = _now
            # calculate average aproximation
            diff = _now - chan.avrg_date
            #period = 30 * time_day
            period = get_config_int('average_period') * time_day
            if not period:
                period = time_day
            avrg_period = chan.avrg_period
            avrg_period += diff
            if avrg_period > period:
                avrg_period = period
            if diff > avrg_period // 1000 and diff > 600:
                # calc average after 1000th part of the period (10 min minimum)
                max = period // 100
                if diff > max:
                    # too much time have passed since last check, average will be skewed by current
                    # user count, so we reduce the average period.
                    excess = diff - max
                    debug('avrg period correction %s e:%.4f%%', args=(key[1],
                        excess*100.0/avrg_period))
                    diff = max
                    avrg_period -= excess
                    if avrg_period < diff:
                        avrg_period = diff
                avrg = chan.average
                avrg = (avrg * (avrg_period - diff) + value * diff) / avrg_period
                chan.avrg_date = _now
                chan.avrg_period = avrg_period
                # make sure avrg is between max and 1
                if avrg > chan.max:
                    avrg = chan.max
                elif avrg < 1:
                    avrg = 1
                debug('avrg %s %.2f → %.2f (%.4f%% %.4f)', args=(key[1], chan.average, avrg,
                    diff*100.0/avrg_period, avrg - chan.average))
                chan.average = avrg
        else:
            CaseInsensibleDict.__setitem__(self, key, Channel(count=value))

        #if avrg:
        #    self.logger.log(key, _now, value, avrg)
        #else:
        #    self.logger.log(key, _now, value)

    def initchan(self, key, *args):
        CaseInsensibleDict.__setitem__(self, key, Channel(*args))

    def iterchan(self):
        def generator():
            for key in self.keys():
                chan = self[key]
                row = list(key)
                row.extend(chan)
                yield row

        return generator()

    def keys(self):
        """Returns keys sorted"""
        L = dict.keys(self)
        L.sort()
        return L

    def close(self):
        self.logger.close()

channel_stats = ChanStatDB()


def write_database():
    import csv
    filename = get_dir('peak_data.csv')
    try:
        writer = csv.writer(open(filename, 'wb'))
        writer.writerows(channel_stats.iterchan())
    except IOError:
        error('Failed to write chanstat database in %s' %file)

def load_database():
    import csv
    filename = get_dir('peak_data.csv')
    try:
        reader = csv.reader(open(filename, 'rb'))
    except IOError:
        return
    channel_stats.clear()
    for row in reader:
        key = tuple(row[0:2])
        values = row[2:-1]
        values = map(int, values)
        average = row[-1]
        average = float(average)
        values.append(average)
        channel_stats.initchan(key, *values)

def update_user_count(server=None, channel=None):
    if isinstance(channel, str):
        channel = set((channel, ))
    elif channel:
        channel = set(channel)

    def update_channel(server, channel=None):
        channel_infolist = weechat.infolist_get('irc_channel', '', server)
        while weechat.infolist_next(channel_infolist):
            _channel = weechat.infolist_string(channel_infolist, 'name')
            if channel:
                _channel = caseInsensibleKey(_channel)
                if _channel not in channel:
                    continue
            channel_stats[server, _channel] = weechat.infolist_integer(channel_infolist, 'nicks_count')
        weechat.infolist_free(channel_infolist)

    if not server:
        server_infolist = weechat.infolist_get('irc_server', '', '')
        while weechat.infolist_next(server_infolist):
            server = weechat.infolist_string(server_infolist, 'name')
            update_channel(server)
        weechat.infolist_free(server_infolist)
    else:
        update_channel(server, channel)

def time_elapsed(elapsed, ret=None, level=2):
    if ret is None:
        ret = []

    if not elapsed:
        return ''

    if elapsed > time_year:
        years, elapsed = elapsed // time_year, elapsed % time_year
        ret.append('%s%s' %(years, 'y'))
    elif elapsed > time_day:
        days, elapsed = elapsed // time_day, elapsed % time_day
        ret.append('%s%s' %(days, 'd'))
    elif elapsed > time_hour:
        hours, elapsed = elapsed // time_hour, elapsed % time_hour
        ret.append('%s%s' %(hours, 'h'))
    elif elapsed > 60:
        mins, elapsed = elapsed // 60, elapsed % 60
        ret.append('%s%s' %(mins, 'm'))
    else:
        secs, elapsed = elapsed, 0
        ret.append('%s%s' %(secs, 's'))

    if len(ret) >= level or not elapsed:
        return ' '.join(ret)

    ret = time_elapsed(elapsed, ret, level)
    return ret

channel_peak_hooks = CaseInsensibleDict()
msg_queue_timeout = 15
def new_channel_peak(key, count, time=0):
    if not get_config_boolean('show_peaks'):
        return
    if key in channel_peak_hooks:
        weechat.unhook(channel_peak_hooks[key][0])
        time = channel_peak_hooks[key][1]
    else:
        time -= 60 * msg_queue_timeout # add delay in showing the msg

    if time:
        elapsed = time_elapsed(now() - time)
        if elapsed:
            elapsed = '(last peak was %s ago)' %elapsed
    else:
        elapsed = ''

    # hook it for show msg 10 min later
    channel_peak_hooks[key] = (weechat.hook_timer(60000 * msg_queue_timeout, 0, 1, 'new_channel_peak_cb',
            '%s,%s,New user peak: %s users %s' %(key[0], key[1], count, elapsed)), time)

def new_channel_peak_cb(data, count):
    debug(data)
    server, channel, s = data.split(',', 2)
    buffer = weechat.info_get('irc_buffer', '%s,%s' %(server, channel))
    if buffer:
        say('%s%s' %(color_peak, s), buffer=buffer)
    else:
        debug('XX falied to get buffer: %s.%s', args=(server, channel))
    del channel_peak_hooks[server, channel]
    return WEECHAT_RC_OK

channel_low_hooks = CaseInsensibleDict()
def new_channel_low(key, count, time=0):
    if not get_config_boolean('show_lows'):
        return
    if key in channel_low_hooks:
        weechat.unhook(channel_low_hooks[key][0])
        time = channel_low_hooks[key][1]
    else:
        time -= 60 * msg_queue_timeout # add delay in showing the msg

    if time:
        elapsed = time_elapsed(now() - time)
        if elapsed:
            elapsed = '(last low was %s ago)' %elapsed
    else:
        elapsed = ''

    # hook it for show msg 10 min later
    channel_low_hooks[key] = (weechat.hook_timer(60000 * msg_queue_timeout, 0, 1, 'new_channel_low_cb',
            '%s,%s,New user low: %s %s' %(key[0], key[1], count, elapsed)), time)

def new_channel_low_cb(data, count):
    debug(data)
    server, channel, s = data.split(',', 2)
    buffer = weechat.buffer_search('irc', '%s.%s' %(server, channel))
    if buffer:
        say('%s%s' %(color_low, s), buffer=buffer)
    else:
        debug('XX falied to get buffer: %s.%s', args=(server, channel))
    del channel_low_hooks[server, channel]
    return WEECHAT_RC_OK

# chanstat command
def chanstat_cmd(data, buffer, args):
    if args == '--save':
        write_database()
        channel_stats.close()
        say('Channel statistics saved.')
        return WEECHAT_RC_OK
    elif args == '--load':
        load_database()
        say('Channel statistics loaded.')
        return WEECHAT_RC_OK
#    elif args == '--print':
#        prnt = weechat.command

    channel = weechat.buffer_get_string(buffer, 'localvar_channel')
    server = weechat.buffer_get_string(buffer, 'localvar_server')
    key = (server, channel)

    update_user_count(server, channel)
    # clear any update in queue
    if key in update_channel_hook:
        weechat.unhook(update_channel_hook[key][0])
        del update_channel_hook[key]

    try:
        chan = channel_stats[server, channel]
        _now = now()
        peak_time = time_elapsed(_now - chan.max_date)
        low_time = time_elapsed(_now - chan.min_date)
        if peak_time:
            peak_time = ' (%s ago)' %peak_time
        if low_time:
            low_time = ' (%s ago)' %low_time
        if chan.avrg_period > time_hour:
            average = ' average: %s%.2f%s users (%s period)' %(color_avg, chan.average,
                    color_reset, time_elapsed(chan.avrg_period, level=1))
        else:
            average = ' (no average yet)'
        say('%s%s%s user peak: %s%s%s%s lowest: %s%s%s%s%s' %(
            color_bold, channel, color_reset,
            color_peak, chan.max, color_reset,
            peak_time,
            color_low, chan.max - chan.min, color_reset,
            low_time, average), buffer=buffer)

        # clear any new peak or low msg in queue
        if key in channel_peak_hooks:
            weechat.unhook(channel_peak_hooks[key][0])
            del channel_peak_hooks[key]
        if key in channel_low_hooks:
            weechat.unhook(channel_low_hooks[key][0])
            del channel_low_hooks[key]
    except KeyError:
        say('No statistics available.', buffer=buffer)

    return WEECHAT_RC_OK

def cmd_debug(data, buffer, args):
    dbg = lambda s, a: debug(s, args=a, name_suffix='dump')
    dbg('\nStats DB: %s', len(channel_stats))
    for key in channel_stats.keys():
        dbg('%s %s - %s', (key[0], key[1], channel_stats[key]))

    dbg('\nUsers: %s', len(domain_list))
    return WEECHAT_RC_OK

class Queue(dict):
    """User queue, for ignore many joins from same host (clones)"""
    def __contains__(self, key):
        self.clear()
        return dict.__contains__(self, key)

    def clear(self):
        _now = now()
        for key, time in self.items():
            if (_now - time) > 60:
                #debug('clearing domain %s from list (count: %s)' %(key, len(self)))
                del self[key]

domain_list = Queue()


# signal callbacks
def join_cb(data, signal, signal_data):
    #debug('%s  %s\n%s' %(data, signal, signal_data), 'SIGNAL')
    global netsplit
    if netsplit:
        debug('ignoring, netsplit')
        if (now() - netsplit) > 30*60: # wait 30 min
            netsplit = 0
        return WEECHAT_RC_OK

    server = signal[:signal.find(',')]
    signal_data = signal_data.split()
    channel = signal_data[2].strip(':')
    host = signal_data[0].strip(':')
    domain = '%s,%s' %(channel, host[host.find('@')+1:])
    if domain in domain_list:
        #debug('ignoring %s', args=domain)
        return WEECHAT_RC_OK
    else:
        domain_list[domain] = now()
    debug(' -- ping %s (%s)', args=(channel,signal[-4:]))
    add_update_user_hook(server, channel)
    return WEECHAT_RC_OK

netsplit = 0
def quit_cb(data, signal, signal_data):
    #debug('%s  %s\n%s' %(data, signal, signal_data), 'SIGNAL')
    global netsplit
    if netsplit:
        return WEECHAT_RC_OK
    quit_msg = signal_data[signal_data.rfind(':')+1:]
    if quit_msg_is_split(quit_msg):
        netsplit = now()
        for hook, when in update_channel_hook.itervalues():
            weechat.unhook(hook)
        update_channel_hook.clear()
        debug('NETSPLIT')

    return WEECHAT_RC_OK

def quit_msg_is_split(s):
    #if 'peer' in s: return True
    if s.count(' ') == 1:
        sp = s.find(' ')
        d1 = s.find('.')
        d2 = s.rfind('.')
        if 0 < d1 and 4 < d2 and d1 < sp < d2 and d2 + 1 < len(s):
            return True
    return False

update_channel_hook = CaseInsensibleDict()
update_queue_timeout = 120
def add_update_user_hook(server, channel):
    key = (server, channel)
    if key in update_channel_hook:
        hook, when = update_channel_hook[key]
        if (now() - when) > update_queue_timeout//2:
            debug(' vv rescheduling %s', args=key[1])
            weechat.unhook(hook)
        else:
            return
    else:
        debug(' >> scheduling %s', args=key[1])

    # we schedule the channel check for later so we can filter quick joins/parts and netsplits
    update_channel_hook[key] = (weechat.hook_timer(update_queue_timeout * 1000, 0, 1, 'update_user_count_cb',
            ','.join(key)), now())

def update_user_count_cb(data, count):
    server, channel = data.split(',', 1)
    channels = [ chan for serv, chan in update_channel_hook if server == serv ]
    if channels:
        update_user_count(server, channels)
        for chan in channels:
            hook, when = update_channel_hook[server, chan]
            weechat.unhook(hook)
            del update_channel_hook[server, chan]
    return WEECHAT_RC_OK

def script_load():
    load_database()
    update_user_count()

def script_unload():
    write_database()
    channel_stats.close()
    return WEECHAT_RC_OK

# default settings
settings = {
    'path'   :'%h/chanstat',
    'average_period':'30',
    'show_peaks' :'on',
    'show_lows'  :'on',
    }

if __name__ == '__main__' and import_ok and \
        weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
        SCRIPT_DESC, 'script_unload', ''):
    
    # colors
    color_delimiter = weechat.color('chat_delimiters')
    color_chat_nick = weechat.color('chat_nick')
    color_reset     = weechat.color('reset')
    color_peak      = weechat.color('green')
    color_low       = weechat.color('red')
    color_avg       = weechat.color('brown')
    color_bold      = weechat.color('white')

    # pretty [chanop]
    script_nick = '%s[%s%s%s]%s' %(color_delimiter, color_chat_nick, SCRIPT_NAME, color_delimiter,
            color_reset)

    for opt, val in settings.iteritems():
		if not weechat.config_is_set_plugin(opt):
			weechat.config_set_plugin(opt, val)

    script_load()

    weechat.hook_signal('*,irc_in2_join', 'join_cb', '')
    weechat.hook_signal('*,irc_in2_part', 'join_cb', '')
    weechat.hook_signal('*,irc_in2_quit', 'quit_cb', '')

    weechat.hook_command('chanstat', "Display channel's statistics.", '[--save | --load]',
            "Displays channel peak, lowest and average users for current channel.\n"
            "  --save: forces saving the stats database.\n"
            "  --load: forces loading the stats database (Overwriting actual values).\n",
            #" --print: sends /chanstat output to the current channel.",
            '--save|--load', 'chanstat_cmd', '')

    weechat.hook_command('chanstat_debug', '', '', '', '', 'cmd_debug', '')

# vim:set shiftwidth=4 tabstop=4 softtabstop=4 expandtab textwidth=100:



# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 Chris Johnson <raugturi@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

# apply_corrections
#
# A weechat plugin to re-print a user's messages with corrections applied when
# they send a correction string (ex: s/typo/replacement).
#
# The following options are availalble:
#
# check_every: Interval, in seconds, between each check for expired messages.
#              If set to 0 no check will be performed and all messages will be
#              saved indefinitely.
#
# data_timeout: Time, in seconds, before a message is expired. If set to 0
#               messages will never expire.
#
# message_limit: Number of messages to store per nick. If set to 0 all messages
#                will be saved until they expire.
#
# print_format: Format string for the printed corrections.
#               Default: "[nick]: [corrected]"
#
#               Variables allowed:
#
#               [nick]:        The nick of the person who sent the messages.
#               [corrected]:   The corrected text of the previous message(s).
#               [correction]:  The correction (format: s/typo/replacement).
#               [original]:    The original message before correction.
#               [pattern]:     The "typo" portion of the correction.
#               [replacement]: The "replacement" portion of the correction.
#               [timestamp]:   The timestamp of the original message.
#
# print_limit: Maximum number of lines to correct and print to the buffer. If
#              set to 0 all lines that match the pattern will be printed.
#
# Note: Setting check_every, data_timeout, message_limit, or some combination
# thereof to 0's will eat a lot of memory very quickly as it will drastically
# increase the size of the dictionary holding previous messages.  Likewise,
# setting print_limit to 0 with a large number of saved messages will quickly
# fill your screen with a bunch of stuff should someone submit a generic
# enough correction string.

# History:
#
# 2014-05-10, Sébastien Helleu <flashcode@flashtux.org>
#     version 1.2: change hook_print callback argument type of
#                  displayed/highlight (WeeChat >= 1.0)
# 2012-10-09, Chris Johnson <raugturi@gmail.com>:
#     version 1.1: change some more variable names for clarity/consistency
# 2012-10-08, Chris Johnson <raugturi@gmail.com>:
#     version 1.0: fix get_corrected_messages so that the most recent messages
#                  are corrected and returned
# 2012-10-08, Chris Johnson <raugturi@gmail.com>:
#     version 0.9: use defaultdict to handle missing keys, flatten dict by
#                  using (buffer, nick) tuple as key, simplify message logging
#                  logic, rename some stuff for clarity.
# 2012-10-08, Chris Johnson <raugturi@gmail.com>:
#     version 0.8: remove empty buffers and nicks during clean-up
# 2012-09-05, Chris Johnson <raugturi@gmail.com>:
#     version 0.7: fix bug when restoring defaults for options that require
#                  integer values
# 2012-09-05, Chris Johnson <raugturi@gmail.com>:
#     version 0.6: copy info from README into script and shorten the variable
#                  descriptions
# 2012-09-01, Chris Johnson <raugturi@gmail.com>:
#     version 0.5: don't log the reprinted messages
# 2012-08-31, Chris Johnson <raugturi@gmail.com>:
#     version 0.4: use same timestamp as buffer when reprinting, instead
#                  of epoch
# 2012-08-31, Chris Johnson <raugturi@gmail.com>:
#     version 0.3: switch to [var] style variables for print format
# 2012-08-30, Chris Johnson <raugturi@gmail.com>:
#     version 0.2: fixed search for typos so if regex fails it falls back
#                  to string.find
# 2012-08-30, Chris Johnson <raugturi@gmail.com>:
#     version 0.1: initial release

import_ok = True

try:
    import weechat
except ImportError:
    print('This script must be run under WeeChat.')
    print('Get WeeChat now at: http://www.weechat.org/')
    import_ok = False

try:
    import re
    import time
    from operator import itemgetter
    from collections import defaultdict
except ImportError as message:
    print('Missing package(s) for %s: %s' % (SCRIPT_NAME, message))
    import_ok = False

SCRIPT_NAME = 'apply_corrections'
SCRIPT_AUTHOR = 'Chris Johnson <raugturi@gmail.com>'
SCRIPT_VERSION = '1.2'
SCRIPT_LICENSE = 'GPL3'
SCRIPT_DESC = "When a correction (ex: s/typo/replacement) is sent, print the "\
              "user's previous message(s) with the corrected text instead."

# Default settings for the plugin.
settings = {'check_every': '5',
            'data_timeout': '60',
            'message_limit': '2',
            'print_format': '[nick]: [corrected]',
            'print_limit': '1'}

# Initialize the dictionary to store most recent messages per buffer per nick.
LASTWORDS = defaultdict(list)


def apply_correction(message, pattern, replacement):
    """
    Replaces all occurences of pattern in message with replacment.  It tries to
    treat the pattern and replacement as regular expressions, but falls back to
    string replace if that fails.
    """

    try:
        message = re.compile(pattern).sub(replacement, message)
    except:
        message = message.replace(pattern, replacement)

    return message


def get_corrected_messages(nick, log, correction):
    """
    Return list of messages that match the pattern, with corrections applied.
    Limited to print_limit items, sorted by timestamp ascending.
    """

    print_limit = get_option_int('print_limit')
    corrected_messages = []
    pattern, replacement = correction.split('/')[1:3]

    for message in log:
        if print_limit and len(corrected_messages) >= print_limit:
            break
        original = message.get('message', '')
        if original:
            try:
                match = re.match(re.compile('.*%s.*' % pattern), original)
            except:
                match = original.find(pattern) != -1
            finally:
                if match:
                    corrected = apply_correction(original,
                                                 pattern,
                                                 replacement)
                    timeformat = weechat.config_string(
                        weechat.config_get('weechat.look.buffer_time_format'))
                    timestamp = time.strftime(
                        timeformat,
                        time.localtime(float(message['timestamp'])))
                    corrected_messages.append({'nick': nick,
                                               'corrected': corrected,
                                               'correction': correction,
                                               'original': original,
                                               'pattern': pattern,
                                               'replacement': replacement,
                                               'timestamp': timestamp})

    return sorted(corrected_messages, key=itemgetter('timestamp'))


def get_option_int(option):
    """
    Checks to see if a configuration option is an integer and sets it back to
    the default if it isn't.  Returns the value when done.
    """

    try:
        value = int(weechat.config_get_plugin(option))
    except ValueError:
        weechat.config_set_plugin(option, settings[option])
        value = int(weechat.config_get_plugin(option))

    return value


def get_valid_messages(log, expiry):
    """
    Return only the messages that haven't expired.
    """

    valid = []
    for message in log:
        try:
            timestamp = int(message.get('timestamp', 0))
            if timestamp > expiry:
                valid.append(message)
        except ValueError:
            continue

    return valid


def clear_messages_cb(data, remaining_calls):
    """
    Callback that clears old messages from the LASTWORDS dictionary.  The time
    limit is the number of seconds specified in plugin's data_timeout setting.
    If data_timeout is set to 0 then no messages are cleared.
    """

    data_timeout = get_option_int('data_timeout')
    if data_timeout:
        expiry = time.time() - data_timeout
        for buff, nick in LASTWORDS.keys():
            valid_messages = get_valid_messages(LASTWORDS[(buff,nick)], expiry)
            if valid_messages:
                LASTWORDS[(buff, nick)] = valid_messages
            else:
                del LASTWORDS[(buff,nick)]

    return weechat.WEECHAT_RC_OK


def handle_message_cb(data, buffer, date, tags, disp, hl, nick, message):
    """
    Callback that handles new messages.  If the message is in the format of a
    regex find/replace (ex. 's/typo/replacement/', 'nick: s/typo/replacement')
    then the last print_limit messages for that nick are re-printed to the
    current buffer in their oringal order with the change applied.  Otherwise
    the message is stored in LASTWORDS dictionary for this buffer > nick.
    """

    # Don't do anything if the message isn't suppose to be displayed.
    if int(disp):
        buffer_name = weechat.buffer_get_string(buffer, 'name')
        log = LASTWORDS[(buffer_name, nick)]

        # Matches on both 's/typo/replacement' and 'nick: s/typo/replacement',
        # mainly because of bitlbee since it puts your nick in front of
        # incoming messages.
        #
        # Nick regex nicked from colorize_nicks available here:
        # http://www.weechat.org/scripts/source/stable/colorize_nicks.py.html/
        valid_nick = r'([@~&!%+])?([-a-zA-Z0-9\[\]\\`_^\{|\}]+)'
        valid_correction = r's/[^/]*/[^/]*'
        correction_message_pattern = re.compile(
                r'(%s:\s*)?(%s)(/)?$' % (valid_nick, valid_correction))
        match = re.match(correction_message_pattern, message)

        if match:
            # If message is a correction and we have previous messages from
            # this nick, print up to print_limit of the nick's previous
            # messages with corrections applied, in their original order.
            correction = match.group(4)
            if log and correction:
                print_format = weechat.config_get_plugin('print_format')
                for cm in get_corrected_messages(nick, log, correction):
                    corrected_msg = print_format
                    for k, v in cm.iteritems():
                        corrected_msg = corrected_msg.replace('[%s]' % k, v)
                    weechat.prnt_date_tags(buffer, 0, 'no_log', corrected_msg)
        else:
            # If it's not a correction, store the message in LASTWORDS.
            log.insert(0, {'message': message, 'timestamp': date})
            # If there's a per-nick limit, shorten the list to match.
            message_limit = get_option_int('message_limit')
            while message_limit and len(log) > message_limit:
                log.pop()

    return weechat.WEECHAT_RC_OK


def load_config(data=None, option=None, value=None):
    """
    Load configuration options and (re)register hook_timer to clear old
    messages based on the current value of check_every.  If check_every is 0
    then messages are never cleared.
    """

    # On initial load set any unset options to the defaults.
    if not option:
        for option, default in settings.iteritems():
            if not weechat.config_is_set_plugin(option):
                weechat.config_set_plugin(option, default)

    if not option or option.endswith('check_every'):
        # If hook_timer for clearing old messages is set already, clear it.
        old_hook = globals().get('CLEAR_HOOK', None)
        if old_hook is not None:
            weechat.unhook(old_hook)

        # Register hook_timer to clear old messages.
        check_every = get_option_int('check_every') * 1000
        if check_every:
            globals()['CLEAR_HOOK'] = weechat.hook_timer(
                    check_every, 0, 0, 'clear_messages_cb', '')

    return weechat.WEECHAT_RC_OK


def desc_options():
    """
    Load descriptions for all the options.
    """

    weechat.config_set_desc_plugin(
            'check_every', 'Interval between each check for expired messages.')

    weechat.config_set_desc_plugin(
            'data_timeout', 'Time before a message is expired.')

    weechat.config_set_desc_plugin(
            'message_limit', 'Number of messages to store per nick.')

    weechat.config_set_desc_plugin(
            'print_format', 'Format string for the printed corrections.')

    weechat.config_set_desc_plugin(
            'print_limit', 'Maximum number of lines to correct.')

    return weechat.WEECHAT_RC_OK


if __name__ == '__main__' and import_ok:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
                        SCRIPT_LICENSE, SCRIPT_DESC, '', ''):
        # Load the configuration options.
        load_config()

        # Set up the descriptions for each option.
        desc_options()

        # Register hook to run load_config when options are changed.
        weechat.hook_config('plugins.var.python.%s.*' % SCRIPT_NAME,
                            'load_config', '')

        # Register hook_print to process each new message as it comes in.
        weechat.hook_print('', '', '', 1, 'handle_message_cb', '')



# Author and licensing
__Author__ = "Darth-O-Ring"
__Email__ = "darthoring@gmail.com"
__License__ = """
Copyright (C) 2014-2016  Darth-O-Ring   <darthoring@gmail.com>

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""

# Imports
try:
    import weechat

except ImportError:
    import sys

    print '\nError: Script must be run under Weechat.\n'

    sys.exit(2)

import dbus


weechat.register('clemy', "Your mommy's boyfriend", '0.1.1', 'GPLv3', 'Control yo Clementine like boom-blaka!', '', '')

err_message     =       '\nSomething silly just happend.  Make sure Clementine is running mah dude.'


def help():
    """
    Print help menu to main
    weechat buffer.

    """
    weechat.prnt('', '\n            --Start Help--                ')
    weechat.prnt('', '--Option--                     --Command--')
    weechat.prnt('', '-Play                          /clemy play')
    weechat.prnt('', '-Pause                         /clemy pause')
    weechat.prnt('', '-Next                          /clemy next')
    weechat.prnt('', '-Previous                      /clemy prev')
    weechat.prnt('', '-Stop                          /clemy stop')
    weechat.prnt('', '-Play Track <n>                /clemy playtrack <n>')
    weechat.prnt('', '-Volume up by 4%               /clemy vol+')
    weechat.prnt('', '-Volume down by 4%             /clemy vol-')
    weechat.prnt('', '-Volume up by <n>              /clemy vol+by <n>')
    weechat.prnt('', '-Volume down by <n>            /clemy vol-by <n>')
    weechat.prnt('', '-Now Playing                   /clemynp')
    weechat.prnt('', '-Info                          /clemy info')
    weechat.prnt('', '\n            --End Help--                  ')
    return ''



def np():
    """
    Gather artist, song, and album
    info through dbus.

    """
    bus         =       dbus.Bus(dbus.Bus.TYPE_SESSION)
    try:
        bus_object  =       bus.get_object('org.mpris.clementine', '/Player')
        artist_info =       bus_object.GetMetadata()
        artist      =       artist_info['performer'][:]
        album       =       artist_info['album'][:]
        song        =       artist_info['title'][:]
        now_playing =       '{0} - {1} (album: {2})'.format(artist, song, album)

    except (dbus.DBusException, Exception):
        weechat.prnt('', err_message)

        return weechat.WEECHAT_RC_OK

    return now_playing


def process_cb(data, command, rc, out, err):
    process_output      =       ''
    if out != '':
        process_output  +=      out

    if int(rc) > 0:
        weechat.prnt('', process_output)

    return weechat.WEECHAT_RC_OK


def control(data, buffer, args):
    """
    Parse buffer for valid commands
    and build dictionary mapping from
    valid args to system commands.

    """
    args            =       args.split(' ')
    commands        =       {
                            'play'      :   'dbus-send --type=method_call --dest=org.mpris.MediaPlayer2.clementine /org/mpris/MediaPlayer2 org.mpris.MediaPlayer2.Player.PlayPause',
                            'next'      :   'dbus-send --type=method_call --dest=org.mpris.MediaPlayer2.clementine /org/mpris/MediaPlayer2 org.mpris.MediaPlayer2.Player.Next',
                            'prev'      :   'dbus-send --type=method_call --dest=org.mpris.MediaPlayer2.clementine /org/mpris/MediaPlayer2 org.mpris.MediaPlayer2.Player.Previous',
                            'stop'      :   'dbus-send --type=method_call --dest=org.mpris.MediaPlayer2.clementine /org/mpris/MediaPlayer2 org.mpris.MediaPlayer2.Player.Stop',
                            'vol+'      :   'clementine --volume-up',
                            'vol-'      :   'clementine --volume-down',
                            'vol+by'    :   'clementine --volume-increase-by {0}',
                            'vol-by'    :   'clementine --volume-decrease-by {0}',
                            'playtrack' :   'clementine --play-track {0}'
                            }

    try:
        if args[0].lower() == 'play':
            weechat.hook_process(commands['play'], 10 * 1000, 'process_cb', '')
            weechat.prnt('', 'Playing: {0}'.format(np()))

        elif args[0].lower() == 'pause':
            weechat.hook_process(commands['play'], 10 * 1000, 'process_cb', '')
            weechat.prnt('', 'Paused')

        elif args[0].lower() == 'next':
            weechat.hook_process(commands['next'], 10 * 1000, 'process_cb', '')
            weechat.prnt('', 'Playing Next Track Mah Dude...')

        elif args[0].lower() == 'prev':
            weechat.hook_process(commands['prev'], 10 * 1000, 'process_cb', '')
            weechat.prnt('', 'Playing Previous Track Mah Dude...')

        elif args[0].lower() == 'stop':
            weechat.hook_process(commands['stop'], 10 * 1000, 'process_cb', '')
            weechat.prnt('', 'Stopped Playback')

        elif args[0].lower() == 'playtrack':
            weechat.hook_process(commands['playtrack'].format(args[1]), 10 * 1000, 'process_cb', '')
            weechat.prnt('', 'Playing track {0}: {1}'.format(args[1], np()))

        elif args[0].lower() == 'vol+':
            weechat.hook_process(commands['vol+'], 10 * 1000, 'process_cb', '')
            weechat.prnt('', 'Volume Increased By 4%')

        elif args[0].lower() == 'vol-':
            weechat.hook_process(commands['vol-'], 10 * 1000, 'process_cb', '')
            weechat.prnt('', 'Volume Decreased By 4%')

        elif args[0].lower() == 'vol+by':
            weechat.hook_process(commands['vol+by'].format(args[1]), 10 * 1000, 'process_cb', '')
            weechat.prnt('', 'Volume Increased By {0}%'.format(args[1]))

        elif args[0].lower() == 'vol-by':
            weechat.hook_process(commands['vol-by'].format(args[1]), 10 * 1000, 'process_cb', '')
            weechat.prnt('', 'Volume Decreased By {0}%'.format(args[1]))

        elif args[0].lower() == 'help':
            weechat.prnt('', '{0}'.format(help()))

        elif args[0].lower() == 'info':
            weechat.prnt('', 'Currently listenting to: {0}'.format(np()))

        else:
            weechat.prnt('', "\nWhat are you doing dawg?  That's not a valid command!\n")
            help()

    except:
        weechat.prnt('', err_message)

        return ''

    return weechat.WEECHAT_RC_OK


def weechat_np(data, buffer, args):
    """
    Callback function for
    hooked clemynp command.

    """
    weechat.command(buffer, '/me is currently listening to: {0}'.format(np()))

    return weechat.WEECHAT_RC_OK


weechat.hook_command('clemynp', 'Get/output now playing info', '', '', '', 'weechat_np', '')
weechat.hook_command('clemy', 'Control Clementine', "[play] | [pause] | [next] | [prev] | [stop] | [vol+] | [vol-] | [vol+by <n>] | [vol-by <n>] | [playtrack <n>] | [help]",
"""
play:           Play song.
pause:          Pause song.
next:           Play next song.
prev:           Play previous song.
stop:           Stop playback
vol+:           Increase volume by 4%
vol-:           Decrease volume by 4%
vol+by <n>:     Increase volume by n%
vol-by <n>:     Decrease volume by n%
playtrack <n>:  Play track number n
help:  OG help information.

use: /clemynp if you want to print current song to the buffer that the command is launched from.
Get silly with it.

""", '', 'control', '')



# Copyright (c) 2010-2012 by Stephan Huebner <s.huebner@gmx.org>
#
# Intended use:
#
# Set ip-setting to the correct external IP whenever one connects to a server
# (so that dcc-sending works)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# History:
#    v 0.1 - first public release
#    v 0.2 - changed server from which the IP is gathered
#          - switched from Pythons' urlopen to weechats' url transfer
#            (therefore 0.3.7 is required)
#          - some small tweaks in description and (hopefully) and
#            IP-checking, to make sure that I really got a correct one.
#          - changed contact Email to one that actually exists :)
#    v 0.3 - add option to change service from which the IP is gathered
#          - better recognition of ipv4 addresses and support of ipv6
#          - add mute option
#    v 0.4 - check if xfer plugin is loaded.

SCR_NAME    = "xfer_setip"
SCR_AUTHOR  = "Stephan Huebner <shuebnerfun01@gmx.org>"
SCR_VERSION = "0.4"
SCR_LICENSE = "GPL3"
SCR_DESC    = "Set apropriate xfer-option for external ip"
SCR_COMMAND = "xfer_setip"

import_ok = True
ip_from_option = 0

OPTIONS         = { 'mute'      : ('off','hide output'),
                    'url'       : ('http://checkip.dyndns.org/','url to fetch'),
                  }

try:
    import weechat as w
    from HTMLParser import HTMLParser
    import re

except:
    print "Script must be run under weechat. http://www.weechat.org"
    import_ok = False

def alert(myString):
    w.prnt("", myString)
    return

# create a subclass and override the handler methods
class MyHTMLParser(HTMLParser):
    def handle_data(self, data):
        global OPTIONS, ip_from_option

        data=data.strip()

        ipv6 = re.compile(r"""
        \s*                         # Leading whitespace
        (?!.*::.*::)                # Only a single whildcard allowed
        (?:(?!:)|:(?=:))            # Colon iff it would be part of a wildcard
        (?:                         # Repeat 6 times:
            [0-9a-f]{0,4}           #   A group of at most four hexadecimal digits
            (?:(?<=::)|(?<!::):)    #   Colon unless preceeded by wildcard
        ){6}                        #
        (?:                         # Either
            [0-9a-f]{0,4}           #   Another group
            (?:(?<=::)|(?<!::):)    #   Colon unless preceeded by wildcard
            [0-9a-f]{0,4}           #   Last group
            (?: (?<=::)             #   Colon iff preceeded by exacly one colon
             |  (?<!:)              #
             |  (?<=:) (?<!::) :    #
             )                      # OR
         |                          #   A v4 address with NO leading zeros
            (?:25[0-4]|2[0-4]\d|1\d\d|[1-9]?\d)
            (?: \.
                (?:25[0-4]|2[0-4]\d|1\d\d|[1-9]?\d)
            ){3}
        )
        \s*                         # Trailing whitespace
        $
    """, re.VERBOSE | re.IGNORECASE | re.DOTALL)

        ipv4 = re.compile('(([2][5][0-5]\.)|([2][0-4][0-9]\.)|([0-1]?[0-9]?[0-9]\.)){3}'
                +'(([2][5][0-5])|([2][0-4][0-9])|([0-1]?[0-9]?[0-9]))')

        matchipv4 = ipv4.search(data)
        matchipv6 = ipv6.search(data)
        set_ip = ""
        current_ip = ""

        if matchipv4:
            current_ip = matchipv4.group()
            set_ip = "/set xfer.network.own_ip %s" % matchipv4.group()
        if matchipv6:
            current_ip = matchipv6.group()
            set_ip = "/set xfer.network.own_ip %s" % matchipv6.group()
        if OPTIONS['mute'].lower() == "on":
            set_ip = "/mute %s" % set_ip

        if set_ip != "" and current_ip != ip_from_option:
            w.command("", set_ip)

def fn_setip(data, command, return_code, out, err):
    if return_code != w.WEECHAT_HOOK_PROCESS_ERROR:
        parser.feed(out)
    return w.WEECHAT_RC_OK

def fn_connected(data, signal, signal_data):
    global ip_from_option
    # check if xfer option exists
    own_ip_option = w.config_get("xfer.network.own_ip")
    if not own_ip_option:
        return w.WEECHAT_RC_OK
    ip_from_option = w.config_string(own_ip_option)
    w.hook_process('url:%s' % OPTIONS['url'], 60000, "fn_setip", "")
    return w.WEECHAT_RC_OK

def fn_command(data, buffer, args):
    fn_connected("", "", "")
    return w.WEECHAT_RC_OK

# ================================[ weechat options & description ]===============================
def init_options():
    for option,value in OPTIONS.items():
        if not w.config_is_set_plugin(option):
            w.config_set_plugin(option, value[0])
            OPTIONS[option] = value[0]
        else:
            OPTIONS[option] = w.config_get_plugin(option)
        w.config_set_desc_plugin(option, '%s (default: "%s")' % (value[1], value[0]))

def toggle_refresh(pointer, name, value):
    global OPTIONS
    option = name[len('plugins.var.python.' + SCR_NAME + '.'):]        # get optionname
    OPTIONS[option] = value                                               # save new value
    return w.WEECHAT_RC_OK

if __name__ == "__main__" and import_ok:
    if w.register(SCR_NAME, SCR_AUTHOR, SCR_VERSION, SCR_LICENSE,
                  SCR_DESC, "", ""):
        init_options()
        w.hook_config('plugins.var.python.' + SCR_NAME + '.*', 'toggle_refresh', '')
        parser = MyHTMLParser()
        fn_connected("", "", "")
        # hook to "connected to a server"-signal
        w.hook_signal("irc_server_connected", "fn_connected", "")
        w.hook_command( # help-text
           SCR_COMMAND, "",
"""

The script tries to retrieve your external IP, in three cases:
  1) When it is loaded
  2) whenever you have succesfully connected to a server
  3) when the script is called as a command ("/xfer_setip").

""",
           "",
           "", "fn_command", ""
           )



# -*- coding: utf-8 -*-
#
# Copyright (c) 2016 by Ferus Castor <ferus+weechat@airmail.cc>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#


# Select a buffer from dmenu or rofi
# To call externally (IE: from i3), enable weechat fifo and run:
#  $ echo "core.weechat */buffer_dmenu" >> $(find ~/.weechat -type p)
#
# Optionally requires i3-py [py2] (or i3ipc [py3]) to focus weechat in i3
#
# History:
# 2017-05-03, Ferus
#     version 0.1.1: fix argument error for config_set_plugin
# 2016-05-01, Ferus
#     version 0.1: initial release - requires WeeChat ≥ 0.3.7

# TODO:
#   Option to remove certain buffer types
#   Implement `focus` for other window managers
#   if buffer == currentbuffer: switch to previous buffer

#pylint: disable=I0011,W0603,W1401

SCRIPT_NAME    = "buffer_dmenu"
SCRIPT_AUTHOR  = "Ferus <ferus+weechat@airmail.cc>"
SCRIPT_VERSION = "0.1.1"
SCRIPT_LICENSE = "GPL3"
SCRIPT_DESC    = "List buffers in dmenu (or rofi), changes active window to selected buffer"
SCRIPT_COMMAND = "buffer_dmenu"

import os
import sys
import subprocess

try:
	import weechat as w
except ImportError as e:
	print("This script must be run under WeeChat.")
	exit(1)

try:
	if sys.version_info[0] == 3:
		import i3ipc as i3
	else:
		import i3
	have_i3 = True
except ImportError as e:
	have_i3 = False

settings = {"launcher": ("dmenu", "launcher to use (supported: dmenu/rofi)")
	,"focus"           : ("false", "whether to immediately focus the terminal after selecting buffer")
	,"focus.wm"        : ("i3", "wm focus logic to use (supported: i3)")
	,"dmenu.command"   : ("dmenu -b -i -l 20", "command used to call dmenu")
	,"rofi.command"    : ("rofi -p '# ' -dmenu -lines 10 -columns 8 -auto-select -mesg '<big>Pick a <b>buffer</b> to jump to:</big>'", "command used to call rofi")
	,"title.regex"     : ("WeeChat \d+\.\d+", "regex used to match weechat's title window")
}


def check_dmenu():
	devnull = open(os.devnull)
	retcode = subprocess.call(["which", "dmenu"], stdout=devnull, stderr=devnull)
	return True if retcode == 0 else False


def check_rofi():
	devnull = open(os.devnull)
	retcode = subprocess.call(["which", "rofi"], stdout=devnull, stderr=devnull)
	return True if retcode == 0 else False


def get_launcher():
	launcher = w.config_get_plugin("launcher")
	command = None
	if launcher == "dmenu":
		if check_dmenu():
			command = w.config_get_plugin("dmenu.command")
	elif launcher == "rofi":
		if check_rofi():
			command = w.config_get_plugin("rofi.command")
	return command


def launch(options):
	launcher = get_launcher()
	if launcher:
		call(launcher, options)
	return True


def focus():
	if w.config_string_to_boolean(w.config_get_plugin("focus")):
		if w.config_get_plugin("focus.wm") == "i3":
			focus_i3()


def focus_i3():
	if have_i3:
		regex = w.config_get_plugin("title.regex")

		#py3
		if sys.version_info[0] == 3:
			i3conn = i3.Connection()
			weechat = i3conn.get_tree().find_named(regex)[0]
			weechat.command('focus')

		#py2
		else:
			i3.focus(title=regex)


def call(command, options):
	options = "\n".join(options).encode("utf-8")

	w.hook_process_hashtable("sh"
		,{"arg1": "-c","arg2": 'echo "{0}" | {1}'.format(options, command)}
		,10 * 1000
		,"launch_process_cb"
		,""
	)


process_output = ""
def launch_process_cb(data, command, rc, out, err):
	global process_output
	if out != "":
		process_output += out
	if int(rc) >= 0:
		selected = process_output.decode("utf-8").strip("\n")
		number, name = selected.split(":")
		process_output = ""
		switch_to_buffer(name)
		focus()
	return w.WEECHAT_RC_OK


def get_open_buffers():
	buffers = []
	infolist = w.infolist_get("buffer", "", "")
	if infolist:
		while w.infolist_next(infolist):
			name = w.infolist_string(infolist, "name")
			number = w.infolist_integer(infolist, "number")
			_ = "{0}:{1}".format(number, name).decode("utf-8")
			buffers.append(_)
		w.infolist_free(infolist)
	return buffers


def get_hotlist_buffers():
	buffers = []
	infolist = w.infolist_get("hotlist", "", "")
	if infolist:
		while w.infolist_next(infolist):
			number = w.infolist_integer(infolist, "buffer_number")
			buffer = w.infolist_pointer(infolist, "buffer_pointer")
			name = w.buffer_get_string(buffer, "name")
			_ = "{0}:{1}".format(number, name).decode("utf-8")
			buffers.append(_)
		w.infolist_free(infolist)
	return buffers


def switch_to_buffer(buffer_name):
	w.command("", "/buffer {0}".format(buffer_name))


def dmenu_cmd_cb(data, buffer, args):
	""" Command /buffers_dmenu """
	if args == "hotlist":
		buffers = get_hotlist_buffers()
	else:
		buffers = get_open_buffers()

	if not launch(buffers):
		return w.WEECHAT_RC_ERROR
	return w.WEECHAT_RC_OK


if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE, SCRIPT_DESC, "", ""):
	version = w.info_get('version_number', '') or 0
	for option, value in settings.items():
		if not w.config_is_set_plugin(option):
			w.config_set_plugin(option, value[0])
		if int(version) >= 0x00030500:
			w.config_set_desc_plugin(option, '{0} (default: "{1}")'.format(value[1], value[0]))

	w.hook_command(SCRIPT_COMMAND
		,"Show a list of all buffers in dmenu"
		,"[hotlist]"
		,"  hotlist: shows hotlist buffers only\n"
		"\n"
		"To call externally (IE: from i3), enable weechat fifo and run:\n"
		"  $ echo 'core.weechat */buffer_dmenu' >> $(find ~/.weechat -type p)\n"
		"\n"
		"To focus the terminal containing WeeChat for the following WM:\n"
		"  i3: requires i3-py [py2] (or i3ipc [py3]) from pip\n"
		,""
		,"dmenu_cmd_cb"
		,""
	)



# -*- coding: utf-8 -*-
# vim: ai ts=4 sts=4 et sw=4 nu


from __future__ import (unicode_literals, absolute_import,
                        division, print_function)


import weechat as w
import paho.mqtt.client as mqtt
import json

SCRIPT_NAME = 'mqtt_notify'
SCRIPT_AUTHOR = 'Guillaume Subiron <maethor@subiron.org>'
SCRIPT_VERSION = '0.2'
SCRIPT_LICENSE = 'WTFPL'
SCRIPT_DESC = 'Sends notifications using MQTT'

w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
           SCRIPT_DESC, '', '')

DEFAULT_OPTIONS = {
    'mqtt_host': 'localhost',
    'mqtt_port': '1883',
    'mqtt_timeout': '60',
    'mqtt_user': '',
    'mqtt_password': '',
    'mqtt_channel': 'weechat',
}

for key, val in DEFAULT_OPTIONS.items():
    if not w.config_is_set_plugin(key):
        w.config_set_plugin(key, val)

w.hook_print("", "notify_message", "", 1, "on_msg", "")
w.hook_print("", "notify_private", "", 1, "on_msg", "private")
w.hook_print("", "notify_highlight", "", 1, "on_msg", "")  # Not sure if needed


def on_msg(*a):
    keys = ['data', 'buffer', 'timestamp', 'tags', 'displayed', 'highlight',
            'sender', 'message']
    msg = dict(zip(keys, a))
    msg['buffer'] = w.buffer_get_string(msg['buffer'], 'short_name')

    cli = mqtt.Client()
    if w.config_get_plugin('mqtt_user'):
        cli.username_pw_set(w.config_get_plugin('mqtt_user'),
                            password=w.config_get_plugin('mqtt_password'))
    cli.connect(w.config_get_plugin('mqtt_host'),
                int(w.config_get_plugin('mqtt_port')),
                int(w.config_get_plugin('mqtt_timeout')))
    cli.publish(w.config_get_plugin('mqtt_channel'),
                json.dumps(msg), retain=True)

    return w.WEECHAT_RC_OK



# -*- coding: utf-8 -*-
#
# Copyright (C) 2003-2012 Sebastien Helleu <flashcode@flashtux.org>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
# Display name days in bar item and buffer.
# Currently, only french calendar is supported.
# (this script requires WeeChat 0.3.0 or newer)
#
# History:
#
# 2012-02-02, Sebastien Helleu <flashcode@flashtux.org>:
#     version 1.3: add option "reminder"
# 2012-01-03, Sebastien Helleu <flashcode@flashtux.org>:
#     version 1.2: make script compatible with Python 3.x
# 2011-10-30, Sebastien Helleu <flashcode@flashtux.org>:
#     version 1.1: fix colors in output of /nameday
# 2011-05-06, Sebastien Helleu <flashcode@flashtux.org>:
#     version 1.0: add some missing names and color based on gender
# 2010-01-14, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.9: add color options and options to display dates in bar item
# 2010-01-13, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.8: conversion to python (script renamed to nameday.py),
#                  conversion to WeeChat 0.3.0+
# 2007-08-10, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.7
# 2003-12-06, Sebastien Helleu <flashcode@flashtux.org>:
#     version 0.1: initial release (fete.pl)
#

SCRIPT_NAME    = 'nameday'
SCRIPT_AUTHOR  = 'Sebastien Helleu <flashcode@flashtux.org>'
SCRIPT_VERSION = '1.3'
SCRIPT_LICENSE = 'GPL3'
SCRIPT_DESC    = 'Display name days in bar item and buffer'

SCRIPT_COMMAND  = 'nameday'
SCRIPT_BAR_ITEM = 'nameday'

import_ok = True

try:
    import weechat
except ImportError:
    print('This script must be run under WeeChat.')
    print('Get WeeChat now at: http://www.weechat.org/')
    import_ok = False

try:
    import sys, time, unicodedata, re
    from datetime import date
except ImportError as message:
    print('Missing package(s) for %s: %s' % (SCRIPT_NAME, message))
    import_ok = False

# script options
nameday_settings_default = {
    'country'                : ('fr',      'country, only "fr" (french) is currently available'),
    'days'                   : ('1',       'number of days after current one to display in bar item'),
    'item_date_today'        : ('on',      'display date for today in bar item'),
    'item_date_next'         : ('off',     'display dates for tomorrow and next days in bar item'),
    'item_name_gender'       : ('off',     'display gender (St/Ste) before name'),
    'item_color_date_today'  : ('white',   'color for date in item (today)'),
    'item_color_name_today'  : ('green',   'color for name in item (today)'),
    'item_color_date_next'   : ('default', 'color for date in item (next days)'),
    'item_color_name_next'   : ('default', 'color for name in item (next days)'),
    'item_color_male_today'  : ('cyan',    'color for male names in item (today)'),
    'item_color_female_today': ('magenta', 'color for female names in item (today)'),
    'item_color_male_next'   : ('cyan',    'color for male names in item (next days)'),
    'item_color_female_next' : ('magenta', 'color for female names in item (next days)'),
    'color_male'             : ('cyan',    'color for male names'),
    'color_female'           : ('magenta', 'color for female names'),
    'reminder'               : ('',        'comma-separated list of names or dates (format: DD/MM) for which a reminder is displayed'),
}
nameday_settings = {}

nameday_item = ''
nameday_buffer = ''

namedays = {
    'fr': (
        # january
        ('!Marie (JOUR DE L\'AN)', '&Basile', '!Geneviève', '&Odilon', '&Edouard',
         '&Mélaine', '&Raymond,&Cédric,!Virginie', '&Lucien', '!Alix', '&Guillaume',
         '&Paulin', '!Tatiana', '!Yvette', '!Nina', '&Rémi,!Rachel',
         '&Marcel', '!Roseline', '!Prisca', '&Marius', '&Sébastien',
         '!Agnès', '&Vincent', '&Barnard', '&François de Sales', 'Conversion de St Paul',
         '!Paule', '!Angèle', '&Thomas d\'Aquin', '&Gildas', '!Martine,!Jacinthe',
         '!Marcelle'),
        # february
        ('!Ella,&Siméon', 'Présentation,&Théophane', '&Blaise', '!Véronique', '!Agathe',
         '&Gaston,!Dorothée', '!Eugénie', '!Jacqueline', '!Apolline', '&Arnaud',
         'Notre-Dame de Lourdes', '&Félix', '!Béatrice', '&Valentin', '&Claude,&Jordan',
         '!Julienne,!Lucile', '&Alexis', '!Bernadette', '&Gabin', '!Aimée',
         '&Damien', '!Isabelle', '&Lazare', '&Modeste', '&Roméo',
         '&Nestor', '!Honorine', '&Romain', '&Auguste'),
        # march
        ('&Aubin,&Albin,&Jonathan', '&Charles le Bon', '&Guénolé,&Marin', '&Casimir', '!Olive,!Olivia',
         '&Colette', '!Félicité,&Nathan', '&Ryan', '!Françoise', '&Vivien',
         '!Rosine', '!Justine,&Pol', '&Rodrigue', '!Mathilde', '!Louise',
         '!Bénédicte', '&Patrice,&Patrick', '&Cyrille', '&Joseph', '&Herbert',
         '!Clémence,!Axelle', '!Léa', '&Victorien', '!Catherine', '&Humbert',
         '!Larissa', '&Habib', '&Gontran', '!Gwladys', '&Amédée',
         '&Benjamin'),
        # april
        ('&Hugues,&Valéry', '!Sandrine', '&Richard', '&Isodore', '!Irène',
         '&Marcellin', '&Jean-Baptiste de la Salle,&Clotaire', '!Julie', '&Gautier', '&Fulbert',
         '&Stanislas', '&Jules', '!Ida', '&Maxime,!Ludivine', '&César,&Paterne',
         '&Benoît-Joseph Labre', '&Anicet', '&Parfait', '!Emma', '!Odette',
         '&Anselme', '&Alexandre', '&Georges', '&Fidèle', '&Marc',
         '!Alida', '!Zita', '!Valérie', '!Catherine de Sienne', '&Robert'),
        # may
        ('&Jérémie (FETE du TRAVAIL)', '&Boris,!Zoé', '&Philippe,&Jacques', '&Sylvain,&Florian', '!Judith',
         '!Prudence', '!Gisèle', '&Désiré (ANNIVERSAIRE 1945)', '&Pacôme', '!Solange',
         '!Estelle', '&Achille', '!Rolande,&Maël', '&Mathias,!Aglaé', '!Denise',
         '&Honoré,&Brendan', '&Pascal', '&Eric,!Corinne', '&Yves,&Erwan', '&Bernardin',
         '&Constantin', '&Emile,!Rita', '&Didier', '&Donatien', '!Sophie',
         '&Bérenger', '&Augustin', '&Germain', '&Aymar,!Géraldine', '&Ferdinand,!Jeanne',
         'Pétronille'),
        # june
        ('&Justin,&Ronan', '!Blandine', '&Kévin', '!Clotilde', '&Igor',
         '&Norbert', '&Gilbert', '&Médard', '!Diane', '&Landry',
         '&Barnabé,!Yolande', '&Guy', '&Antoine de Padoue', '&Elisée,&Valère', '!Germaine',
         '&François-Régis,&Régis', '&Hervé', '&Léonce', '&Romuald,&Gervais,!Micheline', '&Silvère',
         '&Rodolphe', '&Alban', '!Audrey', '&Jean-Baptiste', '&Salomon,&Prosper,!Aliénor,!Eléonore',
         '&Anthelme', '&Fernand', '&Irénée', '&Pierre,&Paul', '&Martial,&Adolphe'),
        # july
        ('&Thierry,!Esther', '&Martinien', '&Thomas', '&Florent', '&Antoine',
         '!Nolwen,!Mariette', '&Raoul', '&Thibaut,&Edgar,&Kilian,!Priscilla', '!Amandine,!Hermine,!Marianne', '&Ulrich',
         '&Benoît,!Olga', '&Olivier,&Jason', '&Henri,&Joël,&Enzo,&Eugène', '!Camille (FETE NATIONALE)', '&Donald,&Vladimir',
         '!Elvire', '!Charlotte,!Arlette,!Marcelline', '&Frédéric', '&Arsène', '!Marina',
         '&Victor', '!Madeleine', '!Brigitte', '!Christine,!Ségolène', '&Jacques,!Valentine',
         '!Anne,!Hannah,&Joachin', '!Nathalie', '&Samson', '!Marthe,!Béatrix,&Loup', '!Juliette',
         '&Ignace de Loyola'),
        # august
        ('&Alphonse', '&Julien', '!Lydie', '&Vianney', '&Abel',
         'Transfiguration', '&Gaëtan', '&Dominique', '&Amour', '&Laurent',
         '!Claire,!Gilberte,!Suzanne', '!Clarisse', '&Hippolyte', '&Evrard', '!Marie,&Alfred (ASSOMPTION)',
         '&Armel', '&Hyacinthe', '!Hélène,!Laetitia', '&Jean Eudes', '&Bernard,&Samuel',
         '&Christophe,!Grâce', '&Fabrice', '!Rose de Lima', '&Barthélémy', '&Louis',
         '!Natacha', '!Monique', '&Augustin,&Elouan', '!Sabine,&Médéric', '&Fiacre',
         '&Aristide'),
        # september
        ('&Gilles', '!Ingrid', '&Grégoire', '!Rosalie,!Iris,&Moïse', '!Raïssa',
         '&Bertrand,!Eva', '!Reine', '&Adrien,!Béline', '&Alain,&Omer', '!Inès',
         '&Adelphe,!Glenn,!Vinciane', '&Apollinaire', '&Aimé', 'La Ste Croix', '&Roland,!Dolorès',
         '!Edith', '&Renaud,&Lambert', '!Nadège,!Véra', '!Emilie', '&Davy',
         '&Matthieu,!Déborah', '&Maurice', '&Constant', '!Thècle', '&Hermann',
         '&Côme,&Damien', '&Vincent de Paul', '&Venceslas', '&Michel,&Gabriel,&Raphaël', '&Jérôme'),
        # october
        ('!Thérèse de l\'Enfant Jésus', '&Léger', '&Gérard', '&François d\'Assise', '!Fleur,!Chloé',
         '&Bruno', '&Serge,&Gustave', '!Pélagie,&Thaïs', '&Denis', '&Ghislain,&Virgile',
         '&Firmin', '&Wilfried,&Edwin', '&Géraud', '&Juste,!Céleste,!Gwendoline', '!Thérèse d\'Avila',
         '!Edwige', '&Baudouin,!Solène', '&Luc', '&René,!Cléo', '!Adeline,!Aline',
         '!Céline,!Ursule', '!Elodie,!Salomé', '&Jean de Capistran', '&Florentin', '&Crépin',
         '&Dimitri', '!Emeline', '&Simon,&Jude', '&Narcisse', '!Bienvenue,!Maéva',
         '&Quentin'),
        # november
        ('&Harold (TOUSSAINT)', 'Défunts', '&Hubert,&Gwenaël', '&Charles,&Aymeric', '!Sylvie,&Zacharie',
         '!Bertille,&Léonard', '!Carine', '&Geoffroy', '&Théodore', '&Léon,&Noah,&Noé,!Mélissa',
         '&Martin (ARMISTICE 1918)', '&Christian', '&Brice', '&Sidoine', '&Albert,&Arthur,&Léopold,!Victoire',
         '!Marguerite,!Mégane,!Gertrude', '!Elisabeth,!Elise,!Hilda', '!Aude', '&Tanguy', '&Edmond,&Octave',
         'Présentation de Marie', '!Cécile', '&Clément', '!Flora', '!Catherine',
         '!Delphine', '&Séverin', '&Jacques de la Marche', '&Saturnin', '&André'),
        # december
        ('!Florence', '!Viviane', '&Xavier', '!Barbara', '&Gérald',
         '&Nicolas', '&Ambroise', 'Immaculée Conception', '&Pierre Fourier', '&Romaric',
         '&Daniel', '!Chantal', '!Lucie,&Jocelyn', '!Odile', '!Ninon',
         '!Alice', '&Gaël', '&Gatien', '&Urbain', '&Abraham,&Théophile',
         '&Pierre Canisius', '!Françoise-Xavier', '&Armand', '!Adèle', '&Emmanuel,&Manuel (NOEL)',
         '&Etienne', '&Jean', '&Gaspard', '&David', '&Roger',
         '&Sylvestre,!Colombe'),
        )
    }

nameday_i18n = {
    'fr': { 'm'        : 'St ',
            'f'        : 'Ste ',
            },
    }

def nameday_remove_accents(string):
    """Remove accents from string."""
    if sys.version_info >= (3,):
        # python 3.x
        return unicodedata.normalize('NFKD', string).encode('ASCII', 'ignore').decode('UTF-8')
    else:
        # python 2.x
        return unicodedata.normalize('NFKD', unicode(string, 'UTF-8')).encode('ASCII', 'ignore')

def nameday_get_country():
    """Return country."""
    global nameday_settings, namedays
    country = nameday_settings['country']
    if not namedays[country]:
        country = 'fr'
    return country

def nameday_decode(name, gender, colorMale, colorFemale):
    """Decode name: replace special chars and optionally add color."""
    global nameday_i18n, nameday_settings
    country = nameday_get_country()
    replacement = { '&': '', '!': '' }
    if colorMale:
        replacement['&'] = weechat.color(nameday_settings[colorMale])
    if colorFemale:
        replacement['!'] = weechat.color(nameday_settings[colorFemale])
    if gender:
        replacement['&'] += nameday_i18n[country]['m']
        replacement['!'] += nameday_i18n[country]['f']
    return name.replace('&', replacement['&']).replace('!', replacement['!']).replace(',', ', ')

def nameday_get_month_day(month, day, gender, colorMale, colorFemale):
    """Get name day for given day/month."""
    global namedays
    try:
        country = nameday_get_country()
        name = namedays[country][month][day]
        return nameday_decode(name, gender, colorMale, colorFemale)
    except:
        return ''

def nameday_get_date(name_date, gender, colorMale, colorFemale):
    """Get name day for given date."""
    return nameday_get_month_day(name_date.month - 1, name_date.day - 1, gender, colorMale, colorFemale)

def nameday_completion_namedays_cb(data, completion_item, buffer, completion):
    """Complete with name days, for command '/nameday'."""
    global namedays
    country = nameday_get_country()
    for names in namedays[country]:
        for string in names:
            pos = string.find('(')
            if pos > 0:
                string = string[0:pos].strip()
            for name in string.split(','):
                name2 = nameday_decode(name, gender=False, colorMale='', colorFemale='')
                weechat.hook_completion_list_add(completion,
                                                 name2,
                                                 0, weechat.WEECHAT_LIST_POS_SORT)
                weechat.hook_completion_list_add(completion,
                                                 nameday_remove_accents(name2),
                                                 0, weechat.WEECHAT_LIST_POS_SORT)
    return weechat.WEECHAT_RC_OK

def nameday_buffer_input_cb(data, buffer, input_data):
    """Input callback for buffer."""
    if input_data.lower() == 'q':
        weechat.buffer_close(buffer)
    return weechat.WEECHAT_RC_OK

def nameday_buffer_close_cb(data, buffer):
    """Callback called when buffer is closed."""
    global nameday_buffer
    nameday_buffer = ''
    return weechat.WEECHAT_RC_OK

def nameday_display_list(buffer):
    """Display list of name days in buffer."""
    global namedays
    country = nameday_get_country()
    today = date.today()
    month = 0
    while month < len(namedays[country]):
        day = 0
        while day < len(namedays[country][month]):
            color = ''
            if today.month - 1 == month and today.day - 1 == day:
                color = weechat.color('yellow')
            weechat.prnt(buffer, '%s%02d/%02d  %s' % (color,
                                                      (day + 1), (month + 1),
                                                      nameday_get_month_day(month, day, gender=True, colorMale='color_male', colorFemale='color_female')))
            day += 1
        month += 1

def nameday_list():
    """Open buffer and display list of name days."""
    global nameday_buffer
    if nameday_buffer:
        weechat.buffer_set(nameday_buffer, 'display', '1')
    else:
        nameday_buffer = weechat.buffer_search('python', 'nameday');
        if not nameday_buffer:
            nameday_buffer = weechat.buffer_new('nameday', 'nameday_buffer_input_cb', '',
                                                'nameday_buffer_close_cb', '');
        if nameday_buffer:
            weechat.buffer_set(nameday_buffer, 'localvar_set_no_log', '1')
            weechat.buffer_set(nameday_buffer, 'time_for_each_line', '0')
            weechat.buffer_set(nameday_buffer, 'display', '1')
            weechat.buffer_set(nameday_buffer, 'title', 'Name days  |  Commands: list, listfull')
            nameday_display_list(nameday_buffer)

def nameday_print(days):
    """Print name day for today and option N days in future."""
    global nameday_i18n
    today = date.today()
    current_time = time.time()
    string = '%02d/%02d: %s' % (today.day, today.month,
                                nameday_get_date(today, gender=True,
                                                 colorMale='color_male',
                                                 colorFemale='color_female'))
    if days < 0:
        days = 0
    elif days > 50:
        days = 50
    if days > 0:
        string += '%s (' % weechat.color('reset')
        for i in range(1, days + 1):
            if i > 1:
                string += '%s, ' % weechat.color('reset')
            date2 = date.fromtimestamp(current_time + ((3600 * 24) * i))
            string += '%02d/%02d: %s' % (date2.day, date2.month,
                                         nameday_get_date(date2, gender=True,
                                                          colorMale='color_male',
                                                          colorFemale='color_female'))
        string += '%s)' % weechat.color('reset')
    weechat.prnt('', string)

def nameday_reminder(month=0, day=0, tag='notify_highlight'):
    """Display reminder for given date (or nothing if no reminder defined for today)."""
    global namedays, nameday_settings
    country = nameday_get_country()
    if month < 1 or day < 1:
        today = date.today()
        month = today.month
        day = today.day
    nameday = nameday_remove_accents(namedays[country][month - 1][day - 1]).lower()
    nameday_words = re.sub('[^a-z ]', '', nameday.replace(',', ' ')).split()
    reminder = False
    for name in nameday_settings['reminder'].split(','):
        if name:
            pos = name.find('/')
            if pos >= 0:
                if day == int(name[:pos]) and month == int(name[pos+1:]):
                    reminder = True
                    break
            else:
                wordsfound = True
                for word in name.strip().lower().split():
                    if word and word not in nameday_words:
                        wordsfound = False
                if wordsfound:
                    reminder = True
                    break
    if reminder:
        weechat.prnt_date_tags('', 0, tag,
                               '*\tReminder: %02d/%02d: %s' %
                               (day, month,
                                nameday_get_month_day(month - 1, day - 1,
                                                      gender=True,
                                                      colorMale='color_male',
                                                      colorFemale='color_female')))

def nameday_search(name):
    """Search a name."""
    global namedays
    user_nameday = nameday_remove_accents(name).lower()
    country = nameday_get_country()
    month = 0
    while month < len(namedays[country]):
        day = 0
        while day < len(namedays[country][month]):
            nameday = nameday_remove_accents(namedays[country][month][day])
            if (nameday.lower().find(user_nameday) >= 0):
                weechat.prnt('', '%02d/%02d: %s' % ((day + 1), (month + 1),
                                                    nameday_get_month_day(month, day,
                                                                          gender=True,
                                                                          colorMale='color_male',
                                                                          colorFemale='color_female')))
            day += 1
        month += 1

def nameday_search_reminders():
    """Search and display dates for reminders."""
    global namedays
    country = nameday_get_country()
    month = 0
    while month < len(namedays[country]):
        day = 0
        while day < len(namedays[country][month]):
            nameday_reminder(month + 1, day + 1, '')
            day += 1
        month += 1

def nameday_cmd_cb(data, buffer, args):
    """Command /nameday."""
    if args:
        if args == '*':
            nameday_list()
        elif args == '!':
            nameday_search_reminders()
        elif args.isdigit():
            nameday_print(int(args))
        elif args.find('/') >= 0:
            pos = args.find('/')
            if pos > 0:
                day = int(args[:pos])
                month = int(args[pos+1:])
                name = nameday_get_month_day(month - 1, day - 1, gender=True, colorMale='color_male', colorFemale='color_female')
                if name != '':
                    weechat.prnt('', '%02d/%02d: %s' % (day, month, name))
        else:
            nameday_search(args)
    else:
        nameday_print(1)
        nameday_reminder()
    return weechat.WEECHAT_RC_OK

def nameday_item_cb(data, buffer, args):
    """Callback for building nameday item."""
    global nameday_item
    return nameday_item

def nameday_build_item():
    """Build nameday item."""
    global nameday_settings, nameday_item
    nameday_item = ''
    display_date_today = nameday_settings['item_date_today'].lower() == 'on'
    display_date_next = nameday_settings['item_date_next'].lower() == 'on'
    display_gender = nameday_settings['item_name_gender'].lower() == 'on'
    color_date_today = weechat.color(nameday_settings['item_color_date_today'])
    color_name_today = weechat.color(nameday_settings['item_color_name_today'])
    color_date_next = weechat.color(nameday_settings['item_color_date_next'])
    color_name_next = weechat.color(nameday_settings['item_color_name_next'])
    color_default = weechat.color('default')
    today = date.today()
    if display_date_today:
        nameday_item += '%s%02d/%02d%s: ' % (color_date_today,
                                             today.day, today.month,
                                             color_default)
    nameday_item += '%s%s' % (color_name_today,
                              nameday_get_date(today, gender=display_gender,
                                               colorMale='item_color_male_today',
                                               colorFemale='item_color_female_today'))
    days = 0
    try:
        days = int(nameday_settings['days'])
    except:
        days = 0
    if days < 0:
        days = 0
    if days > 10:
        days = 10
    if days > 0:
        nameday_item += '%s (' % color_default
        current_time = time.time()
        for i in range(1, days + 1):
            if i > 1:
                nameday_item += ', '
            date2 = date.fromtimestamp(current_time + ((3600 * 24) * i))
            if display_date_next:
                nameday_item += '%s%02d/%02d%s: ' % (color_date_next,
                                                     date2.day, date2.month,
                                                     color_default)
            nameday_item += '%s%s' % (color_name_next,
                                      nameday_get_date(date2, gender=display_gender,
                                                       colorMale='item_color_male_next',
                                                       colorFemale='item_color_female_next'))
        nameday_item += '%s)' % color_default
    return nameday_item

def nameday_timer_cb(data, remaining_calls):
    """Called each day at midnight to change item content."""
    nameday_build_item()
    weechat.bar_item_update('nameday')
    nameday_reminder()
    return weechat.WEECHAT_RC_OK

def nameday_load_config():
    global nameday_settings_default, nameday_settings
    version = weechat.info_get('version_number', '') or 0
    for option, value in nameday_settings_default.items():
        if weechat.config_is_set_plugin(option):
            nameday_settings[option] = weechat.config_get_plugin(option)
        else:
            weechat.config_set_plugin(option, value[0])
            nameday_settings[option] = value[0]
        if int(version) >= 0x00030500:
            weechat.config_set_desc_plugin(option, value[1])

def nameday_config_cb(data, option, value):
    """Called each time an option is changed."""
    nameday_load_config()
    nameday_build_item()
    weechat.bar_item_update('nameday')
    return weechat.WEECHAT_RC_OK

if __name__ == '__main__' and import_ok:
    if weechat.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION,
                        SCRIPT_LICENSE, SCRIPT_DESC,
                        '', ''):
        # load config
        nameday_load_config()

        # new command
        weechat.hook_completion('namedays', 'list of name days',
                                'nameday_completion_namedays_cb', '')
        weechat.hook_command(SCRIPT_COMMAND,
                             'Display name days',
                             '[* | number | date | name | !]',
                             '     *: display list of name days in a new buffer\n'
                             'number: display name day for today and <number> days in future\n'
                             '  date: display name day for this date, format is day/month (for example: 31/01)\n'
                             '  name: display date for this name\n'
                             '     !: display reminder dates for names defined in option "reminder"\n\n'
                             'A bar item "nameday" can be used in a bar.\n\n'
                             'Examples:\n'
                             '  /nameday *          display list of name days in a new buffer\n'
                             '  /nameday            display name day for today and tomorrow\n'
                             '  /nameday 2          display name day for today, tomorrow, and after tomorrow\n'
                             '  /nameday 20/01      display name day for january, 20th\n'
                             '  /nameday sébastien  display day for name "sébastien"',
                             '*|!|%(namedays)', 'nameday_cmd_cb', '')

        # new item
        nameday_build_item()
        weechat.bar_item_new(SCRIPT_BAR_ITEM, 'nameday_item_cb', '')

        # timer
        weechat.hook_timer(3600 * 24 * 1000, 3600 * 24, 0, 'nameday_timer_cb', '')

        # config
        weechat.hook_config('plugins.var.python.' + SCRIPT_NAME + '.*', 'nameday_config_cb', '')

        # reminder
        nameday_reminder()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2009 by xt <xt@bash.no>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

#
#
# If someone posts a spotify track URL in a configured channel
# this script will post back which track it is using spotify.url.fi service

#
#
# History:
# 2017-09-30, butlerx
#   version 0.9: Add support for oauth keys being stored in secure data
# 2017-06-02, butlerx
#   version 0.8: add now required oauth support
# 2016-01-22, creadak
#   version 0.7: Updated for the new spotify API
# 2011-03-11, Sebastien Helleu <flashcode@flashtux.org>
#   version 0.6: get python 2.x binary for hook_process (fix problem when
#                python 3.x is default python version)
# 2010-01-12, xt
#   version 0.5: add option to use notice instead of message
# 2009-12-02, xt
#   version 0.4 small bugfix with some songs and popularity
# 2009-10-29, xt
#   version 0.3 use official spotify API, and add support for albums
# 2009-09-25, xt
#   version 0.2: use spotify.url.fi
# 2009-06-19, xt <xt@bash.no>
#     version 0.1: initial
#

import datetime
import re

import spotipy
import weechat as w
from spotipy.oauth2 import SpotifyClientCredentials

SCRIPT_NAME = "spotify"
SCRIPT_AUTHOR = "xt <xt@bash.no>"
SCRIPT_VERSION = "0.9"
SCRIPT_LICENSE = "GPL"
SCRIPT_DESC = "Look up spotify urls"

settings = {
    "buffers": 'freenode.#mychan,',  # comma separated list of buffers
    "emit_notice": 'off',  # on or off, use notice or msg
    "client_id": 'client_id',
    "client_secret": 'client_secret'
}

settings_help = {
    "buffers":
    'A comma separated list of buffers the script should check',
    "emit_notice":
    'If on, this script will use /notice, if off, it will use /msg to post info',
    "client_id":
    'required client id token go to https://developer.spotify.com/my-applications/#!/applications to generate your own',
    "client_secret":
    'required client secret token go to https://developer.spotify.com/my-applications/#!/applications to generate your own'
}

spotify_track_res = (
    re.compile(r'spotify:(?P<type>\w+):(?P<id>\w{22})'),
    re.compile(r'https?://open.spotify.com/(?P<type>\w+)/(?P<id>\w{22})'))


def get_spotify_ids(s):
    for r in spotify_track_res:
        for type, track in r.findall(s):
            yield type, track


def get_oauth(arg):
    token = w.config_get_plugin(arg)
    if token.startswith('${sec.data'):
        return w.string_eval_expression(token, {}, {}, {})
    else:
        return token


def parse_response(data, type):
    if type == 'track':
        name = data['name']
        album = data['album']['name']
        artist = data['artists'][0]['name']
        duration = str(
            datetime.timedelta(milliseconds=data['duration_ms'])).split('.')[0]
        popularity = data['popularity']
        return "%s - %s / %s %s %d%%" % (artist, name, album, duration,
                                         popularity)
    elif type == 'album':
        name = data['name']
        artist = data['artists'][0]['name']
        tracks = data['tracks']['total']
        released = data['release_date'].split('-')[0]
        length = 0
        for track in data['tracks']['items']:
            length += track['duration_ms']
        duration = str(datetime.timedelta(milliseconds=length)).split('.')[0]
        return "%s - %s (%s) - %d tracks (%s)" % (artist, name, released,
                                                  tracks, duration)
    elif type == 'artist':
        name = data['name']
        followers = data['followers']['total']
        return "%s - %s followers" % (name, followers)


def spotify_print_cb(data, buffer, time, tags, displayed, highlight, prefix,
                     message):
    notice = w.config_get_plugin('emit_notice')
    buffer_name = w.buffer_get_string(buffer, "name")
    server, channel = buffer_name.split('.')
    buffers_to_check = w.config_get_plugin('buffers').split(',')
    client_credentials_manager = SpotifyClientCredentials(
        get_oauth('client_id'), get_oauth('client_secret'))
    spotify = spotipy.Spotify(
        client_credentials_manager=client_credentials_manager)

    command = "msg"
    if notice == "on":
        command = "notice"

    if buffer_name.lower() not in [
            buffer.lower() for buffer in buffers_to_check
    ]:
        return w.WEECHAT_RC_OK

    for type, id in get_spotify_ids(message):
        if type == 'album':
            results = spotify.album(id)
        elif type == 'track':
            results = spotify.track(id)
        elif type == 'artist':
            results = spotify.artist(id)
        reply = parse_response(results, type)
        w.command('', "/%s -server %s %s %s" % (command, server, channel,
                                                reply))
    return w.WEECHAT_RC_OK


if __name__ == "__main__":
    if w.register(SCRIPT_NAME, SCRIPT_AUTHOR, SCRIPT_VERSION, SCRIPT_LICENSE,
                  SCRIPT_DESC, "", ""):
        # Set default settings
        for option, default in settings.iteritems():
            if not w.config_is_set_plugin(option):
                w.config_set_plugin(option, default)

        # Set help text
        for option, description in settings_help.iteritems():
            w.config_set_desc_plugin(option, description)

        w.hook_print("", "", "spotify", 1, "spotify_print_cb", "")



# This script renames your Facebook buddies to a readable format when
# using Facebook's XMPP gateway with Bitlbee or Minbif.

# Based on the Irssi script at http://browsingtheinternet.com/temp/bitlbee_rename.txt
# Ported for Weechat 0.3.0 or later by Jaakko Lintula (crwl@iki.fi)
# Modified for Minbif by 'varogami' <varogami@gmail.com>
#
# This script is in the public domain.


# Edit this variables with your own minbif/bitlbee configuration settings
fullname="Full Name"       # set the first word in full name string that "/WHOIS nick nick" or /WII show
                           # italian - fullname="Nome"
                           # spanish - fullname="Nombre completo"
                           # english - fullname="Full Name"
myChannel = "&minbif" # set "&bitlbee" or "&minbif" if you use default settings
myServer = "minbif" # set "bitlbee" or "minbif" if you use default settings

facebookhostname = "chat.facebook.com"
gmailhostname = "public.talk.google.com"

myBuffer = "%s.%s" % (myServer, myChannel)
nicksToRename = set()

import weechat
import re

weechat.register("im_rename", "crwl", "1.2", "Public Domain", "Renames Facebook anf Google Plus usernames with Minbif", "", "")

def message_join_minbif(data, signal, signal_data):
  signal_data = signal_data.split()
  channel = signal_data[2]
  hostmask = signal_data[0]
  nick = hostmask[1:hostmask.index('!')]
  username = hostmask[hostmask.index('!')+1:hostmask.index('@')]
  server = hostmask[hostmask.index('@')+1:]
  if server.find(':') > 1:
    server = server[:+server.index(':')]

  if channel == myChannel and nick == username and nick[0] == '-' and server == facebookhostname:
   nicksToRename.add(nick)
   weechat.command(weechat.buffer_search("irc", myBuffer), "/whois "+nick+" "+nick)

  if channel == myChannel and nick == "_"+username and nick[0] == '_' and server == gmailhostname:
   nicksToRename.add(nick)
   weechat.command(weechat.buffer_search("irc", myBuffer), "/whois "+nick+" "+nick)

  return weechat.WEECHAT_RC_OK

def whois_data_minbif(data, signal, signal_data):
  if fullname in signal_data:
   nick = signal_data.split(fullname)[0].strip()
   nick = nick[1:nick.index(' :')]
   nick = nick.split(' ')
   nick = nick[3]
   realname =  signal_data.split(fullname)[1].strip()

   if nick in nicksToRename:
     nicksToRename.remove(nick)
     ircname = re.sub("[^A-Za-z0-9]", "", realname)[:24]
     if ircname != nick:
       weechat.command(weechat.buffer_search("irc", myBuffer), "/quote -server %s svsnick %s %s" % (myServer, nick, ircname))

  return weechat.WEECHAT_RC_OK



weechat.hook_signal(myServer+",irc_in_join", "message_join_minbif", "")
weechat.hook_signal(myServer+",irc_in_320", "whois_data_minbif", "")
